from os import path as os_path
import re

from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from DateTime import DateTime

from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.CMFDefault.RegistrationTool import _checkEmail

from Products.CMFPlone.RegistrationTool import RegistrationTool as BaseRegistrationTool

from Globals import package_home, DTMLFile
_dtmldir = os_path.join( package_home( globals() ), 'dtml' )

from config import *

class RegistrationTool(BaseRegistrationTool):

    meta_type = RegistrationToolName

    security = ClassSecurityInfo()

    email = 'nobody@nobody.org'

    min_passwd_lenght = PASSWD_LENGHT

    passwd_pattern = '^.*(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[\W]).*$'
    passwd_expr = re.compile(passwd_pattern)
    passwd_pattern_msg = "Your password must contain at least "\
                         "one digit, one upper character, one lower character "\
                         "and one non alphanumeric character"

    duration = 30 # days


    manage_options = (BaseRegistrationTool.manage_options +
                     (
#                         { 'label' : 'Configure', 'action' : 'manage_configuration' },
                        { 'label' : 'Members', 'action' : 'manage_members' },
                     ) + SimpleItem.manage_options)

    #
    #   ZMI methods
    #
    security.declareProtected(ManagePortal, 'manage_configuration')
    manage_configuration = DTMLFile('configureRegistrationTool', _dtmldir)

    security.declareProtected(ManagePortal, 'manage_members')
    manage_members = DTMLFile('members', _dtmldir)

    security.declareProtected(ManagePortal, 'manage_editSettings')
    def manage_editSettings(self, email, min_passwd_lenght, passwd_pattern, passwd_pattern_msg, duration, REQUEST):
        """ """
        if email:
            self.email = email
        else:
            self.email = 'nobody@nobody.org'

        if min_passwd_lenght:
            self.min_passwd_lenght = int(min_passwd_lenght)
        else:
            self.min_passwd_lenght = PASSWD_LENGHT

        if passwd_pattern:
            self.passwd_pattern = passwd_pattern
            self.passwd_expr = re.compile(self.passwd_pattern)
        else:
            pass

        if passwd_pattern_msg:
            self.passwd_pattern_msg = passwd_pattern_msg
        else:
            pass

        if duration:
            self.duration = int(duration)
        else:
            self.duration = 0

        if REQUEST is not None:
            msg = 'Setting changed'
            return self.manage_configuration(manage_tabs_message=msg)

    security.declarePublic('generatePassword')
    def generatePassword(self):
        """Generates a random password of min lenght"""
        return self.getPassword(self.min_passwd_lenght)

    def _checkFullnameInPassword(self, password, fullname):
        """Check if there is a part of fullname in the password string"""
        return [n for n in fullname.split(' ') if n and n in password]

    security.declarePublic( 'testPasswordValidity' )
    def testPasswordValidity(self, password, confirm=None):

        """ Verify that the password satisfies the portal's requirements.

        o If the password is valid, return None.
        o If not, return a string explaining why.
        """
        if not password:
            return 'You must enter a password.'

        if len(password) < self.min_passwd_lenght and not _checkPermission(ManagePortal, self):
            return 'Your password must contain at least %i characters.'%self.min_passwd_lenght

        if not self.passwd_expr.search(password):
            return self.passwd_pattern_msg

        mtool = getToolByName(self, 'portal_membership')
        member = mtool.getAuthenticatedMember()
        username = member.getId() or self.REQUEST.get('username', '')
        fullname = member.getProperty('fullname') or self.REQUEST.get('fullname', '')

        if username in password or self._checkFullnameInPassword(password, fullname):
            return 'You must avoid to use login or fullname in your password'

        if confirm is not None and confirm != password:
            return ( 'Your password and confirmation did not match. '
                   + 'Please try again.' )

        return None


    security.declareProtected(RESET_PERMISSION, 'resetPassword')
    def resetPassword(self, ids=None, REQUEST=None):
        """ """
        acl_users = getToolByName(self, 'acl_users')
        mtool = getToolByName(self, 'portal_membership')

        if not ids:
            ids = mtool.listMemberIds()

        for id in ids:
            pw = self.generatePassword()
            member = mtool.getMemberById(id)

            acl_users.userFolderEditUser(id, pw, member.getRoles(), member.getDomains())
            member.setMemberProperties({'must_change_password': 1 } )

            if not member.getProperty('email'):
                raise ValueError('%s does not have an email address.'%id)



            check, msg = _checkEmail(member.getProperty('email'))
            if not check:
                raise ValueError, msg

            mail_text = self.mail_password_template( self
                                                  , REQUEST
                                                  , member=member
                                                  , password=pw
                                                  )

            host = self.MailHost
            host.send( mail_text )

            self.enableAccount(member.getId())

        if REQUEST is not None:
            msg = 'Reset password.'
            return self.manage_members(manage_tabs_message=msg)

    def _getMember(self, user_name):
        mtool = getToolByName(self, 'portal_membership')
        return  mtool.getMemberById(user_name)

    def isPasswordExpired(self, user_name):
        """ """
        member = self._getMember(user_name)
        last_password_change = member.getProperty('last_password_change')
        if DateTime() - DateTime(last_password_change)  > self.duration:
            return True
        else:
            return False

    security.declareProtected(ManagePortal, 'checkPasswordAttempts')
    def checkPasswordAttempts(self, user_name):
        """ check and increment password attemps, False if exceedes max (3)"""
        member = self._getMember(user_name)
        password_attempts = member.getProperty('password_attempts', 0)
        if password_attempts > MAX_ATTEMPTS:
            self.disableAccount(user_name)
            return False
        else:
            member.setMemberProperties({'password_attempts': password_attempts + 1 } )
            return True

    security.declareProtected(ManagePortal, 'resetPasswordAttempts')
    def resetPasswordAttempts(self, user_name):
        """ """
        member = self._getMember(user_name)
        member.setMemberProperties({'password_attempts': 0 } )
        print member.getProperty('password_attempts')

    security.declareProtected(ManagePortal, 'disableAccount')
    def disableAccount(self, user_name):
        """ """
        member = self._getMember(user_name)
        member.setMemberProperties({'enabled': False } )

    security.declareProtected(ManagePortal, 'enableAccount')
    def enableAccount(self, user_name):
        """ """
        member = self._getMember(user_name)
        member.setMemberProperties({'enabled': True,
                                    'password_attempts': 0 } )

