import os, sys, re

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
# from Products.CMFPlone.tests import PloneTestCase
from Products.PloneTestCase import PloneTestCase

from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from Products.PrivacyRegistrationTool import RegistrationTool


ZOPE_PRODUCTS = [
  'PrivacyRegistrationTool',
]
for product in ZOPE_PRODUCTS:
    PloneTestCase.installProduct(product)

PLONE_PRODUCTS = [
  'PrivacyRegistrationTool',
]
PloneTestCase.setupPloneSite(products=PLONE_PRODUCTS)

class TestPasswdExpression(ZopeTestCase.ZopeTestCase):

  def setUp(self):

    self.expr = re.compile(RegistrationTool.passwd_pattern)

  def testPasswordExpr(self):

    assert not self.expr.search('abc')
    assert not self.expr.search('ABC')
    assert not self.expr.search('123')

    assert not self.expr.search('aA')
    assert not self.expr.search('a1')
    assert not self.expr.search('a;')
    assert not self.expr.search('Aa')
    assert not self.expr.search('1a')
    assert not self.expr.search(';a')

    assert not self.expr.search('A1')
    assert not self.expr.search('A1;')
    assert not self.expr.search('1A')
    assert not self.expr.search(';1A')

    assert not self.expr.search('Aa0')
    assert not self.expr.search('aA0')
    assert not self.expr.search('0Aa')

    assert self.expr.search('Aa#0')
    assert self.expr.search('aA@0')
    assert self.expr.search(';0Aa')

    assert self.expr.search('123456aB;')
    assert not self.expr.search('12345678')

class TestPasswdMethod(PloneTestCase.PloneTestCase):

    def afterSetUp(self):
        self.tool = getToolByName(self.portal, 'portal_registration')
        mtool = getToolByName(self.portal, 'portal_membership')
        self.member = mtool.getAuthenticatedMember()

    def testMethod(self):

        self.failUnless('You must enter a password.' == self.tool.testPasswordValidity(''))

        self.failUnless('Your password must contain at least' in self.tool.testPasswordValidity('123'))

        self.failUnless(self.tool.passwd_pattern_msg == self.tool.testPasswordValidity('12345678'))

        self.member.setProperties(fullname='firstname lastname')
        self.failUnless('You must avoid to use login or fullname in your password' == self.tool.testPasswordValidity('test_user_1_A$'))
        self.failUnless('You must avoid to use login or fullname in your password' == self.tool.testPasswordValidity('firstnameA$12'))
        self.failUnless('You must avoid to use login or fullname in your password' == self.tool.testPasswordValidity('lastnameA$12'))

        self.member.setProperties(fullname='')
        self.failUnless(not self.tool.testPasswordValidity('aB;45678'))


    def testPasswordExpired(self):

        user_name = self.member.getId()

        self.member.setProperties(last_password_change=DateTime()-self.tool.duration+1)  # expired
        self.failIf(self.tool.isPasswordExpired(user_name))

        self.member.setProperties(last_password_change=DateTime()-self.tool.duration-1)  # not expired
        self.failUnless(self.tool.isPasswordExpired(user_name))

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestPasswdExpression))
    suite.addTest(makeSuite(TestPasswdMethod))
    return suite

if __name__ == '__main__':
    framework()